@@ -7,7 +7,7 @@ module RSpec
     # ExampleGroup and {Example} are the main structural elements of
     # rspec-core. Consider this example:
     #
-    #     describe Thing do
+    #     RSpec.describe Thing do
     #       it "does something" do
     #       end
     #     end
@@ -90,7 +90,7 @@ module RSpec
       # Returns the class or module passed to the `describe` method (or alias).
       # Returns nil if the subject is not a class or module.
       # @example
-      #     describe Thing do
+      #     RSpec.describe Thing do
       #       it "does something" do
       #         described_class == Thing
       #       end
@@ -111,16 +111,14 @@ module RSpec
       #   @overload $1
       #   @overload $1(&example_implementation)
       #     @param example_implementation [Block] The implementation of the example.
-      #   @overload $1(doc_string, *metadata_keys, metadata={})
+      #   @overload $1(doc_string, *metadata)
       #     @param doc_string [String] The example's doc string.
-      #     @param metadata [Hash] Metadata for the example.
-      #     @param metadata_keys [Array<Symbol>] Metadata tags for the example.
-      #       Will be transformed into hash entries with `true` values.
-      #   @overload $1(doc_string, *metadata_keys, metadata={}, &example_implementation)
+      #     @param metadata [Array<Symbol>, Hash] Metadata for the example.
+      #       Symbols will be transformed into hash entries with `true` values.
+      #   @overload $1(doc_string, *metadata, &example_implementation)
       #     @param doc_string [String] The example's doc string.
-      #     @param metadata [Hash] Metadata for the example.
-      #     @param metadata_keys [Array<Symbol>] Metadata tags for the example.
-      #       Will be transformed into hash entries with `true` values.
+      #     @param metadata [Array<Symbol>, Hash] Metadata for the example.
+      #       Symbols will be transformed into hash entries with `true` values.
       #     @param example_implementation [Block] The implementation of the example.
       #   @yield [Example] the example object
       #   @example
@@ -139,6 +137,11 @@ module RSpec
       #     $1 "does something" do |ex|
       #       # ex is the Example object that contains metadata about the example
       #     end
+      #
+      #  @example
+      #     $1 "does something", :slow, :load_factor => 100 do
+      #     end
+      #
       def self.define_example_method(name, extra_options={})
         idempotently_define_singleton_method(name) do |*all_args, &block|
           desc, *args = *all_args
@@ -204,11 +207,10 @@ module RSpec
       #   @overload $1
       #   @overload $1(&example_group_definition)
       #     @param example_group_definition [Block] The definition of the example group.
-      #   @overload $1(doc_string, *metadata_keys, metadata={}, &example_implementation)
+      #   @overload $1(doc_string, *metadata, &example_implementation)
       #     @param doc_string [String] The group's doc string.
-      #     @param metadata [Hash] Metadata for the group.
-      #     @param metadata_keys [Array<Symbol>] Metadata tags for the group.
-      #       Will be transformed into hash entries with `true` values.
+      #     @param metadata [Array<Symbol>, Hash] Metadata for the group.
+      #       Symbols will be transformed into hash entries with `true` values.
       #     @param example_group_definition [Block] The definition of the example group.
       #
       #   Generates a subclass of this example group which inherits
@@ -223,12 +225,21 @@ module RSpec
       #         do_something_before
       #       end
       #
+      #       before(:example, :clean_env) do
+      #         env.clear!
+      #       end
+      #
       #       let(:thing) { Thing.new }
       #
       #       $1 "attribute (of something)" do
       #         # examples in the group get the before hook
       #         # declared above, and can access `thing`
       #       end
+      #
+      #       $1 "needs additional setup", :clean_env, :implementation => JSON do
+      #         # specifies that hooks with matching metadata
+      #         # should be be run additionally
+      #       end
       #     end
       #
       # @see DSL#describe
@@ -424,11 +435,15 @@ module RSpec
           superclass.method(:next_runnable_index_for),
           description, *args, &example_group_block
         )
+
+        config = RSpec.configuration
+        config.apply_derived_metadata_to(@metadata)
+
         ExampleGroups.assign_const(self)
 
         @currently_executing_a_context_hook = false
 
-        RSpec.configuration.configure_group(self)
+        config.configure_group(self)
       end
 
       # @private
@@ -587,6 +602,7 @@ module RSpec
 
         should_run_context_hooks = descendant_filtered_examples.any?
         begin
+          RSpec.current_scope = :before_context_hook
           run_before_context_hooks(new('before(:context) hook')) if should_run_context_hooks
           result_for_this_group = run_examples(reporter)
           results_for_descendants = ordering_strategy.order(children).map { |child| child.run(reporter) }.all?
@@ -599,6 +615,7 @@ module RSpec
           RSpec.world.wants_to_quit = true if reporter.fail_fast_limit_met?
           false
         ensure
+          RSpec.current_scope = :after_context_hook
           run_after_context_hooks(new('after(:context) hook')) if should_run_context_hooks
           reporter.example_group_finished(self)
         end
@@ -686,6 +703,7 @@ module RSpec
         end
       end
 
+      # @private
       def initialize(inspect_output=nil)
         @__inspect_output = inspect_output || '(no description provided)'
         super() # no args get passed
@@ -746,8 +764,9 @@ module RSpec
                 "on an example group (e.g. a `describe` or `context` block)."
         end
 
-        super
+        super(name, *args)
       end
+      ruby2_keywords :method_missing if respond_to?(:ruby2_keywords, true)
     end
     # rubocop:enable Metrics/ClassLength
 
@@ -766,6 +785,7 @@ module RSpec
       # @return [String] the location where the shared example was included
       attr_reader :inclusion_location
 
+      # @private
       def initialize(shared_group_name, inclusion_location)
         @shared_group_name  = shared_group_name
         @inclusion_location = inclusion_location
